/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.qpid.server.model.testmodels.hierarchy;

import java.util.Map;

import com.google.common.util.concurrent.Futures;
import com.google.common.util.concurrent.ListenableFuture;

import org.apache.qpid.server.model.AbstractConfiguredObject;
import org.apache.qpid.server.model.ConfiguredObject;
import org.apache.qpid.server.model.ManagedAttributeField;
import org.apache.qpid.server.model.State;
import org.apache.qpid.server.model.StateTransition;
import org.apache.qpid.server.model.testmodels.TestSecurityManager;
import org.apache.qpid.server.security.SecurityManager;

public class TestAbstractEngineImpl<X extends TestAbstractEngineImpl<X>> extends AbstractConfiguredObject<X> implements TestEngine<X>
{
    private final TestSecurityManager _securityManager;
    @ManagedAttributeField
    private ListenableFuture<Void> _beforeCloseFuture = Futures.immediateFuture(null);

    @ManagedAttributeField
    private Object _stateChangeFuture = Futures.immediateFuture(null);

    @ManagedAttributeField
    private RuntimeException _stateChangeException;

    public TestAbstractEngineImpl(final Map<Class<? extends ConfiguredObject>, ConfiguredObject<?>> parents,
                                  final Map<String, Object> attributes)
    {
        super(parents, attributes);
        _securityManager = new TestSecurityManager(parents.get(TestCar.class));
    }

    @Override
    public Object getBeforeCloseFuture()
    {
        return _beforeCloseFuture;
    }

    @Override
    public void setBeforeCloseFuture(final ListenableFuture<Void> listenableFuture)
    {
        _beforeCloseFuture = listenableFuture;
    }

    @Override
    public Object getStateChangeFuture()
    {
        return _stateChangeFuture;
    }

    @Override
    public void setStateChangeFuture(final ListenableFuture<Void> listenableFuture)
    {
        _stateChangeFuture = listenableFuture;
    }


    @Override
    public Object getStateChangeException()
    {
        return _stateChangeException;
    }

    @Override
    public void setStateChangeException(final RuntimeException exception)
    {
        _stateChangeException = exception;
    }

    @Override
    protected ListenableFuture<Void> beforeClose()
    {
        return _beforeCloseFuture;
    }

    @StateTransition(currentState = {State.UNINITIALIZED, State.ERRORED}, desiredState = State.ACTIVE)
    private ListenableFuture<Void> onActivate()
    {
        RuntimeException stateChangeException = _stateChangeException;
        if (stateChangeException != null)
        {
            throw stateChangeException;
        }
        return (ListenableFuture<Void>) _stateChangeFuture;
    }

    @StateTransition(currentState = {State.ACTIVE, State.UNINITIALIZED, State.ERRORED}, desiredState = State.DELETED)
    private ListenableFuture<Void> doDelete()
    {
        // to invoke ACO#unregisterChild as part of delete
        deleted();
        return Futures.immediateFuture(null);
    }

    @Override
    protected SecurityManager getSecurityManager()
    {
        return _securityManager;
    }
}
